@@ -11,7 +11,10 @@ from functools import reduce, wraps
 from importlib import import_module
 from inspect import currentframe
 from itertools import chain, product
+from locale import getlocale
 from math import log
+from pkgutil import iter_modules
+from platform import system
 from random import randint
 from six import binary_type, string_types, text_type, BytesIO
 from string import *
@@ -33,9 +36,14 @@ except ImportError:  # Python 3
 __all__ = ["add", "add_macro", "add_map", "b", "clear", "codecs", "decode", "encode", "ensure_str", "examples", "guess",
            "isb", "generate_strings_from_regex", "get_alphabet_from_mask", "handle_error", "is_native",
            "list_categories", "list_encodings", "list_macros", "lookup", "maketrans", "os", "rank", "re", "register",
-           "remove", "reset", "s2i", "search", "stopfunc", "BytesIO", "MASKS", "PY3", "_input", "_stripl", "CodecMacro"]
+           "remove", "reset", "s2i", "search", "stopfunc", "BytesIO", "_input", "_stripl", "CodecMacro",
+           "DARWIN", "LANG", "LINUX", "MASKS", "PY3", "UNIX", "WINDOWS"]
 CODECS_REGISTRY = None
 CODECS_CATEGORIES = ["native", "custom"]
+try:
+    LANG = getlocale()[0][:2].lower()
+except TypeError:
+    LANG = None
 MASKS = {
     'a': printable,
     'b': "".join(chr(i) for i in range(256)),
@@ -47,13 +55,18 @@ MASKS = {
     's': " ",
     'u': ascii_uppercase,
 }
-PY3 = sys.version[0] == "3"
+
 __codecs_registry = []
 
 MACROS = {}
 PERS_MACROS = {}
 PERS_MACROS_FILE = os.path.expanduser("~/.codext-macros.json")
 
+DARWIN = system() == "Darwin"
+LINUX = system() == "Linux"
+PY3 = sys.version[0] == "3"
+UNIX = DARWIN or LINUX
+WINDOWS = system() == "Windows"
 
 entropy = lambda s: -sum([p * log(p, 2) for p in [float(s.count(c)) / len(s) for c in set(s)]])
 
@@ -191,25 +204,31 @@ def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=
         fenc, fdec, name = encode, decode, encoding
         # prepare CodecInfo input arguments
         if pattern:
-            m = re.match(pattern, encoding)
+            m, args, i = re.match(pattern, encoding), [], 1
             try:
-                g = m.group(1) or ""
-                if g.isdigit():
-                    g = int(g)
-                fenc = fenc(g) if fenc else fenc
-                fdec = fdec(g) if fdec else fdec
-            except AttributeError:
-                # this occurs when m is None or there is an error in fenc(g) or fdec(g), meaning no match
-                if m is not None:
-                    raise
-                return
+                while True:
+                    try:
+                        g = m.group(i) or ""
+                        if g.isdigit():
+                            g = int(g)
+                        args += [g]
+                        i += 1
+                    except AttributeError:
+                        # this occurs when m is None or there is an error in fenc(g) or fdec(g), meaning no match
+                        if m is not None:
+                            raise
+                        return
             except IndexError:
-                # this occurs while m is not None, but possibly no capture group that gives at least 1 group index ; in
-                #  this case, if fenc/fdec is a decorated function, execute it with no arg
-                if fenc and len(getfullargspec(fenc).args) == 1:
-                    fenc = fenc()
-                if fdec and len(getfullargspec(fdec).args) == 1:
-                    fdec = fdec()
+                # this occurs while m is not None, but possibly no capture group that gives at least 1 group index ;
+                #  in this case, if fenc/fdec is a decorated function, execute it with no arg
+                if len(args) == 0:
+                    if fenc and len(getfullargspec(fenc).args) == 1:
+                        fenc = fenc()
+                    if fdec and len(getfullargspec(fdec).args) == 1:
+                        fdec = fdec()
+                else:
+                    fenc = fenc(*args) if fenc else fenc
+                    fdec = fdec(*args) if fdec else fdec
         if fenc:
             fenc = fix_inout_formats(fenc)
         if fdec:
@@ -273,10 +292,11 @@ def add(ename, encode=None, decode=None, pattern=None, text=True, add_to_codecs=
             CODECS_CATEGORIES.append(cat)
         ci.parameters['category'] = kwargs.get('category', cat)
         ci.parameters['examples'] = kwargs.get('examples', glob.get('__examples__'))
-        ci.parameters['guess'] = kwargs.get('guess', glob.get('__guess__', [ename]))
+        ci.parameters['guess'] = kwargs.get('guess', glob.get('__guess__', [ename])) or []
         ci.parameters['module'] = kwargs.get('module', glob.get('__name__'))
         ci.parameters.setdefault("scoring", {})
-        for attr in ["bonus_func", "entropy", "len_charset", "penalty", "printables_rate", "padding_char"]:
+        for attr in ["bonus_func", "entropy", "expansion_factor", "len_charset", "penalty", "printables_rate",
+                     "padding_char"]:
             a = kwargs.pop(attr, None)
             if a is not None:
                 ci.parameters['scoring'][attr] = a
@@ -1037,6 +1057,7 @@ def generate_strings_from_regex(regex, star_plus_max=STAR_PLUS_MAX, repeat_max=R
 
 
 # guess feature objects
+__module_exists = lambda n: n in [x[1] for x in iter_modules()]
 stopfunc = ModuleType("stopfunc", """
     Predefined stop functions
     ~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1046,33 +1067,91 @@ stopfunc = ModuleType("stopfunc", """
     - `flag`:       searches for the pattern "[Ff][Ll1][Aa4@][Gg9]" (either UTF-8 or UTF-16)
     - `lang_**`:    checks if the given lang (any from the PROFILES_DIRECTORY of the langdetect module) is detected
     - `printables`: checks that every output character is in the set of printables
+    - `regex`:      takes one argument, the regular expression, for checking a string against the given pattern
+    - `text`:       checks for printables and an entropy less than 4.6 (empirically determined)
 """)
 stopfunc.printables = lambda s: all(c in printable for c in ensure_str(s))
-stopfunc.regex      = lambda p: lambda s: re.search(p, ensure_str(s)) is not None
-stopfunc.text       = lambda s: stopfunc.printables(s) and entropy(s) < 4.6
+stopfunc.printables.__name__ = stopfunc.printables.__qualname__ = "printables"
+stopfunc.regex = lambda p: lambda s: re.search(p, ensure_str(s)) is not None
+stopfunc.regex.__name__ = stopfunc.regex.__qualname__ = "regex"
+stopfunc.text = lambda s: stopfunc.printables(s) and entropy(s) < 4.6
+stopfunc.text.__name__ = stopfunc.text.__qualname__ = "text"
+stopfunc.flag = lambda x: re.search(r"[Ff][Ll1][Aa4@][Gg96]", ensure_str(x)) is not None
+stopfunc.flag.__name__ = stopfunc.flag.__qualname__ = "flag"
+stopfunc.default = stopfunc.text
+
+stopfunc.LANG_BACKEND = None
+stopfunc.LANG_BACKENDS = [n for n in ["langid", "langdetect", "pycld2", "cld3", "textblob"] if __module_exists(n)]
+if len(stopfunc.LANG_BACKENDS) > 0:
+    stopfunc.LANG_BACKEND = stopfunc.LANG_BACKENDS[0]
+if "cld3" in stopfunc.LANG_BACKENDS:
+    stopfunc.CLD3_LANGUAGES = "af|am|ar|bg|bn|bs|ca|ce|co|cs|cy|da|de|el|en|eo|es|et|eu|fa|fi|fr|fy|ga|gd|gl|gu|ha|" \
+                              "hi|hm|hr|ht|hu|hy|id|ig|is|it|iw|ja|jv|ka|kk|km|kn|ko|ku|ky|la|lb|lo|lt|lv|mg|mi|mk|" \
+                              "ml|mn|mr|ms|mt|my|ne|nl|no|ny|pa|pl|ps|pt|ro|ru|sd|si|sk|sl|sm|sn|so|sq|sr|st|su|sv|" \
+                              "sw|ta|te|tg|th|tr|uk|ur|uz|vi|xh|yi|yo|zh|zu".split("|")
+if "textblob" in stopfunc.LANG_BACKENDS:
+    stopfunc.TEXTBLOB_LANGUAGES = "af|ar|az|be|bg|bn|ca|cs|cy|da|de|el|en|eo|es|et|eu|fa|fi|fr|ga|gl|gu|hi|hr|ht|hu|" \
+                                  "id|is|it|iw|ja|ka|kn|ko|la|lt|lv|mk|ms|mt|nl|no|pl|pt|ro|ru|sk|sl|sq|sr|sv|sw|ta|" \
+                                  "te|th|tl|tr|uk|ur|vi|yi|zh".split("|")
+
+
+def _detect(text):
+    _lb, t = stopfunc.LANG_BACKEND, ensure_str(text)
+    if _lb is None:
+        raise ValueError("No language backend %s" % ["selected", "installed"][len(stopfunc.LANG_BACKENDS) == 0])
+    return langid.classify(t)[0] if _lb == "langid" else \
+           langdetect.detect(t) if _lb == "langdetect" else \
+           pycld2.detect(t)[2][0][1] if _lb == "pycld2" else \
+           cld3.get_language(t).language[:2] if _lb == "cld3" else \
+           textblob.TextBlob(t).detect_language()[:2]
+
 
 def _lang(lang):
     def _test(s):
         if not stopfunc.text(s):
             return False
         try:
-            return detect(ensure_str(s)) == lang
+            return _detect(ensure_str(s))[:2] == lang
         except:
             return False
     return _test
 
-try:
-    from langdetect import detect, PROFILES_DIRECTORY
-    for lang in [p.replace("-", "") for p in os.listdir(PROFILES_DIRECTORY)]:
-        setattr(stopfunc, "lang_%s" % lang, _lang(lang))
-except ImportError:
-    pass
 
-
-__flag = lambda x: re.search(r"[Ff][Ll1][Aa4@][Gg96]", x) is not None
-def _flag(x):
-    return __flag(ensure_str(x))
-stopfunc.flag = _flag
+def _load_lang_backend(backend=None):
+    # import the requested backend library if not imported yet
+    if backend is None or backend in stopfunc.LANG_BACKENDS:
+        stopfunc.LANG_BACKEND = backend
+        if backend:
+            globals()[backend] = __import__(backend)
+    else:
+        raise ValueError("Unsupported language detection backend")
+    # remove language-related stop functions
+    for attr in dir(stopfunc):
+        if attr.startswith("_") or not isinstance(getattr(stopfunc, attr), FunctionType):
+            continue
+        if re.match(r"lang_[a-z]{2}$", attr):
+            delattr(stopfunc, attr)
+    # rebind applicable language-related stop functions
+    if stopfunc.LANG_BACKEND:
+        _lb = stopfunc.LANG_BACKEND
+        if _lb == "langid":
+            langid.langid.load_model()
+        for lang in (
+            langid.langid.identifier.nb_classes if _lb == "langid" else \
+            list(set(p[:2] for p in os.listdir(langdetect.PROFILES_DIRECTORY))) if _lb == "langdetect" else \
+            list(set(x[1][:2] for x in pycld2.LANGUAGES if x[0] in pycld2.DETECTED_LANGUAGES)) if _lb == "pycld2" else \
+            stopfunc.CLD3_LANGUAGES if _lb == "cld3" else \
+            stopfunc.TEXTBLOB_LANGUAGES if _lb == "textblob" else \
+            []):
+            n = "lang_%s" % lang
+            setattr(stopfunc, n, _lang(lang))
+            getattr(stopfunc, n).__name__ = getattr(stopfunc, n).__qualname__ = n
+        if LANG:
+            flng = "lang_%s" % LANG
+            if getattr(stopfunc, flng, None):
+                stopfunc.default = getattr(stopfunc, flng)        
+_load_lang_backend(stopfunc.LANG_BACKEND)
+stopfunc._reload_lang = _load_lang_backend
 
 
 def __develop(encodings):
@@ -1132,7 +1211,7 @@ def __guess(prev_input, input, stop_func, depth, max_depth, min_depth, codec_cat
         if encoding in e:
             continue
         if debug:
-            print("[*] Depth %d/%d ; trying %s" % (depth+1, max_depth, encoding))
+            print("[*] Depth %0{}d/%d: %s".format(len(str(max_depth))) % (depth+1, max_depth, encoding))
         __guess(input, new_input, stop_func, depth+1, max_depth, min_depth, codec_categories, exclude, result,
                 found + (encoding, ), stop, show, scoring_heuristic, extended, debug)
 
@@ -1148,6 +1227,8 @@ def __rank(prev_input, input, codecs, heuristic=False, extended=False, yield_sco
 
 
 class _Text(object):
+    __slots__ = ["entropy", "lcharset", "len", "padding", "printables"]
+    
     def __init__(self, text, pad_char=None):
         self.len = len(text)
         self.lcharset = len(set(text))
@@ -1193,13 +1274,24 @@ def __score(prev_input, input, codec, heuristic=False, extended=False):
             elif not pad and obj.padding:
                 s -= .1  # it could arise a padding character is encountered while not being padding => small penalty
             # give a bonus when the rate of printable characters is greater or equal than expected and a penalty when
-            #  lower only for codecs that tolerate errors (otherwise, the printables rate can be biased)
+            #  lower only for codecs that DO NOT tolerate errors (otherwise, the printables rate can be biased)
             if not ci.parameters.get('no_error', False):
                 pr = sc.get('printables_rate', 0)
                 if isinstance(pr, type(lambda: None)):
                     pr = float(pr(obj.printables))
                 if obj.printables - pr <= .05:
                     s += .1
+            expf = sc.get('expansion_factor')
+            if expf:
+                f = float(len(new_input)) / obj.len
+                if isinstance(expf, type(lambda: None)):
+                    expf = expf(f)
+                elif isinstance(expf, (int, float)):
+                    epxf = f - .1 <= expf <= f + .1
+                elif isinstance(expf, (tuple, list)) and len(expf) == 2:
+                    expf = f - expf[1] <= expf[0] <= expf[1] + .1
+                if expf:
+                    s += .1
             # afterwards, if the input text has an entropy close to the expected one, give a bonus weighted on the
             #  number of input characters to take bad entropies of shorter strings into account
             entr = sc.get('entropy', {})
@@ -1211,7 +1303,7 @@ def __score(prev_input, input, codec, heuristic=False, extended=False):
                     entr = entr(obj.entropy)
             if entr is not None:
                 # use a quadratic heuristic to compute a weight for the entropy delta, aligned on (100w,.1) and (200w,1)
-                d_entr = min(4e-05 * obj.len**2 - .003 * obj.len, 1) * abs(entr - obj.entropy)
+                d_entr = min(4e-05 * obj.len**2 - .003 * obj.len, 1) * abs(entr - entropy(new_input))
                 if d_entr <= .5:
                     s += .5 - d_entr
             # finally, if relevant, apply a custom bonus (e.g. when a regex pattern is matched)
@@ -1228,8 +1320,8 @@ def __score(prev_input, input, codec, heuristic=False, extended=False):
             yield s, new_input, encoding
 
 
-def guess(input, stop_func=stopfunc.printables, min_depth=0, max_depth=5, codec_categories=None, exclude=None, found=(),
-          stop=True, show=False, scoring_heuristic=False, extended=False, debug=False):
+def guess(input, stop_func=stopfunc.default, min_depth=0, max_depth=5, codec_categories=None, exclude=None, found=(),
+          stop=True, show=False, scoring_heuristic=True, extended=False, debug=False):
     """ Try decoding without the knowledge of the encoding(s). """
     if max_depth <= 0:
         raise ValueError("Depth must be a non-null positive integer")
@@ -1255,6 +1347,8 @@ codecs.guess = guess
 
 def rank(input, extended=False, limit=-1, codec_categories=None, exclude=None):
     """ Rank the most probable encodings based on the given input. """
+    if isinstance(codec_categories, string_types):
+        codec_categories = (codec_categories, )
     codecs = list_encodings(*(codec_categories or ()))
     for e in __develop(exclude):
         try:
