@@ -16,9 +16,9 @@ __examples__ = {
 }
 
 
-ENCMAP = {k: v for k, v in zip("abeiostz", "48310572")}
+ENCMAP = {k: v for k, v in zip("aabeliostzg", "@4831105729")}
 
 
 add_map("leet", ENCMAP, ignore_case="encode", no_error=True, pattern=r"(?:leet|1337|leetspeak)$",
-        printables_rate=lambda pr: pr)
+        entropy=lambda e: e, expansion_factor=1.)
 
